@@ -5,7 +5,7 @@
 
   MIT License
 
-  Copyright (c) 2017 Michael Truog <mjtruog at gmail dot com>
+  Copyright (c) 2017-2018 Michael Truog <mjtruog at protonmail dot com>
 
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
@@ -37,7 +37,7 @@ module Foreign.Erlang
     , termToBinary
     ) where
 
-import Prelude hiding (length,tail)
+import Prelude hiding (length,tail,(<>))
 import Data.Bits ((.&.))
 import Control.Monad (replicateM)
 import qualified Data.Monoid as Monoid
