@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2011-2017 Michael Truog <mjtruog at gmail dot com>
+// Copyright (c) 2011-2018 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -221,6 +221,35 @@ public class API
         this.subscribe(pattern, new FunctionObject9(instance, methodName));
     }
 
+    /**
+     * Subscribes a static method to a service name pattern.
+     *
+     * @param  pattern     the service name pattern
+     * @param  clazz       the class of the static method
+     * @param  methodName  the static method to handle matching requests
+     *
+     * @throws NoSuchMethodException instance method arity is invalid
+     */
+    public void subscribe(final String pattern,
+                          final Class<?> clazz,
+                          final String methodName)
+        throws NoSuchMethodException
+    {
+        this.subscribe(pattern, new FunctionObject9(this, clazz, methodName));
+    }
+
+    /**
+     * Subscribes a static method to a service name pattern.
+     *
+     * @param  pattern     the service name pattern
+     * @param  callback    method reference for callback (Java 8 or higher)
+     */
+    public void subscribe(final String pattern,
+                          final FunctionInterface10 callback)
+    {
+        this.subscribe(pattern, new FunctionObject10(this, callback));
+    }
+
     /**
      * Subscribes an object method to a service name pattern.
      *
