@@ -24,6 +24,18 @@
 -type socks5_socket() :: {atom(), inet:socket()}.
 -export_type([socks5_socket/0]).
 
+-ifdef(no_proxy_sni_support).
+
+ssl_opts(Host, Opts) ->
+  hackney_connect:ssl_opts(Host, Opts).
+
+-else.
+
+ssl_opts(Host, Opts) ->
+  [{server_name_indication, Host} | hackney_connect:ssl_opts(Host,Opts)].
+
+-endif.
+
 %% @doc Atoms used to identify messages in {active, once | true} mode.
 messages({hackney_ssl, _}) ->
   {ssl, ssl_closed, ssl_error};
@@ -41,14 +53,14 @@ connect(Host, Port, Opts, Timeout) when is_list(Host), is_integer(Port),
   ProxyHost = proplists:get_value(socks5_host, Opts),
   ProxyPort = proplists:get_value(socks5_port, Opts),
   Transport = proplists:get_value(socks5_transport, Opts),
-  
+
   %% filter connection options
   AcceptedOpts =  [linger, nodelay, send_timeout,
     send_timeout_close, raw, inet6],
   BaseOpts = [binary, {active, false}, {packet, 0}, {keepalive,  true},
     {nodelay, true}],
   ConnectOpts = hackney_util:filter_options(Opts, AcceptedOpts, BaseOpts),
-  
+
   %% connect to the socks 5 proxy
   case gen_tcp:connect(ProxyHost, ProxyPort, ConnectOpts, Timeout) of
     {ok, Socket} ->
@@ -56,9 +68,9 @@ connect(Host, Port, Opts, Timeout) when is_list(Host), is_integer(Port),
         ok ->
           case Transport of
             hackney_ssl ->
-              SSlOpts = hackney_connect:ssl_opts(Host, Opts),
+              SSLOpts = ssl_opts(Host, Opts),
               %% upgrade the tcp connection
-              case ssl:connect(Socket, SSlOpts) of
+              case ssl:connect(Socket, SSLOpts) of
                 {ok, SslSocket} ->
                   {ok, {Transport, SslSocket}};
                 Error ->
@@ -186,8 +198,9 @@ do_connection(Socket, Host, Port, Options) ->
   case addr(Host, Port, Resolve) of
     Addr when is_binary(Addr) ->
       ok = gen_tcp:send(Socket, << 5, 1, 0, Addr/binary >>),
-      case gen_tcp:recv(Socket, 10, ?TIMEOUT) of
-        {ok, << 5, 0, 0, BoundAddr/binary >>} ->
+      case gen_tcp:recv(Socket, 4, ?TIMEOUT) of
+        {ok, << 5, 0, 0, AType>>} ->
+          BoundAddr = recv_addr_port(AType, gen_tcp, Socket),
           check_connection(BoundAddr);
         {ok, _} ->
           {error, badarg};
@@ -225,6 +238,19 @@ addr(Host, Port, Resolve) ->
       end
   end.
 
+recv_addr_port(1 = AType, Transport, Socket) -> % IPv4
+   {ok, Data} = Transport:recv(Socket, 6, ?TIMEOUT),
+   <<AType, Data/binary>>;
+recv_addr_port(4 = AType, Transport, Socket) -> % IPv6
+   {ok, Data} = Transport:recv(Socket, 18, ?TIMEOUT),
+   <<AType, Data/binary>>;
+recv_addr_port(3 = AType, Transport, Socket) -> % Domain
+   {ok, <<DLen/integer>>} = Transport:recv(Socket, 1, ?TIMEOUT),
+   {ok, AddrPort} = Transport:recv(Socket, DLen + 2, ?TIMEOUT),
+   <<AType, DLen, AddrPort/binary>>;
+recv_addr_port(_, _, _) ->
+   error.
+
 check_connection(<< 3, _DomainLen:8, _Domain/binary >>) ->
   ok;
 check_connection(<< 1, _Addr:32, _Port:16 >>) ->
