@@ -3,8 +3,6 @@
 %%% This file is part of hackney released under the Apache 2 license.
 %%% See the NOTICE for more information.
 %%%
-%%% Copyright (c) 2012-2014 Benoît Chesneau <benoitc@e-engura.org>
-%%%
 
 -module(hackney_sup).
 
@@ -16,6 +14,8 @@
 %% Supervisor callbacks
 -export([init/1]).
 
+-include("hackney.hrl").
+
 %% Helper macro for declaring children of supervisor
 -define(CHILD(I, Type), {I, {I, start_link, []}, permanent, 5000, Type, [I]}).
 
@@ -28,7 +28,6 @@ start_link() ->
   %% start the pool handler
   PoolHandler = hackney_app:get_app_env(pool_handler, hackney_pool),
   ok = PoolHandler:start(),
-  
   %% finish to start the application
   {ok, Pid}.
 
@@ -37,5 +36,10 @@ start_link() ->
 %% ===================================================================
 
 init([]) ->
+  %% initialize the config table
+  _ = ets:new(?CONFIG, [set, named_table, public]),
+  %% initialize the metric engine
+  hackney_metrics:init(),
   Manager = ?CHILD(hackney_manager, worker),
   {ok, { {one_for_one, 10000, 1}, [Manager]}}.
+
