@@ -6,8 +6,24 @@ defmodule Nodefinder.Mixfile do
 
   def project do
     [app: :nodefinder,
-     version: "1.7.3",
+     version: "1.7.5",
      language: :erlang,
+     erlc_options: [
+       {:d, :erlang.list_to_atom('ERLANG_OTP_VERSION_' ++ :erlang.system_info(:otp_release))},
+       :debug_info,
+       :warnings_as_errors,
+       :strict_validation,
+       :warn_bif_clash,
+       :warn_deprecated_function,
+       :warn_export_all,
+       :warn_export_vars,
+       :warn_exported_vars,
+       :warn_obsolete_guard,
+       :warn_shadow_vars,
+       :warn_unused_import,
+       :warn_unused_function,
+       :warn_unused_record,
+       :warn_unused_vars],
      description: description(),
      package: package(),
      deps: deps()]
