@@ -9,7 +9,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2011-2018 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -29,13 +29,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2011-2017 Michael Truog
-%%% @version 1.7.3 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2011-2018 Michael Truog
+%%% @version 1.7.4 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_configurator).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -behaviour(gen_server).
 
@@ -51,6 +51,7 @@
          services_restart/2,
          services_update/2,
          services_search/3,
+         service_ids/1,
          services/1,
          nodes_set/2,
          nodes_get/1,
@@ -64,13 +65,14 @@
          logging_formatters_set/2,
          logging_redirect_set/2,
          logging/1,
+         code_status/3,
          service_start/2,
          service_stop/3,
          service_restart/2,
          service_update/2,
          service_update_external/5,
-         service_initialized_process/1,
-         service_dead/1]).
+         service_process_init_end/1,
+         service_terminated/1]).
 
 %% gen_server callbacks
 -export([init/1,
@@ -80,6 +82,7 @@
 -include("cloudi_logger.hrl").
 -include("cloudi_core_i_configuration.hrl").
 -include("cloudi_core_i_constants.hrl").
+-include_lib("kernel/include/file.hrl").
 
 -type error_reason_service_start() ::
     {service_internal_module_invalid |
@@ -106,14 +109,21 @@
 -type error_reason_service_update() ::
     {service_internal_update_failed |
      service_external_update_failed, any()}.
+-type error_reason_services_search() ::
+    service_scope_invalid.
+-type error_reason_code_status() ::
+    {file, {file:filename(), file:posix() | badarg}}.
 -export_type([error_reason_service_start/0,
               error_reason_service_stop/0,
               error_reason_service_restart/0,
-              error_reason_service_update/0]).
+              error_reason_service_update/0,
+              error_reason_services_search/0,
+              error_reason_code_status/0]).
 
 -record(state,
     {
-        configuration
+        time_start :: cloudi_timestamp:native_monotonic(),
+        configuration :: #config{}
     }).
 
 -define(CATCH_EXIT(F),
@@ -136,18 +146,14 @@ configure() ->
 
 acl_add(L, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
-                                {acl_add, L,
-                                 timeout_decr(Timeout)}, Timeout)).
+                                {acl_add, L}, Timeout)).
 
 acl_remove(L, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
-                                {acl_remove, L,
-                                 timeout_decr(Timeout)}, Timeout)).
+                                {acl_remove, L}, Timeout)).
 
 acl(Timeout) ->
-    ?CATCH_EXIT(gen_server:call(?MODULE,
-                                {acl,
-                                 timeout_decr(Timeout)}, Timeout)).
+    ?CATCH_EXIT(gen_server:call(?MODULE, acl, Timeout)).
 
 service_subscriptions(ServiceId, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
@@ -156,92 +162,80 @@ service_subscriptions(ServiceId, Timeout) ->
 
 services_add(L, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
-                                {services_add, L,
-                                 timeout_decr(Timeout)}, Timeout)).
+                                {services_add, L}, Timeout)).
 
 services_remove(L, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
-                                {services_remove, L,
-                                 timeout_decr(Timeout)}, Timeout)).
+                                {services_remove, L}, Timeout)).
 
 services_restart(L, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
-                                {services_restart, L,
-                                 timeout_decr(Timeout)}, Timeout)).
+                                {services_restart, L}, Timeout)).
 
 services_update(L, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
-                                {services_update, L,
-                                 timeout_decr(Timeout)}, Timeout)).
+                                {services_update, L}, Timeout)).
 
 services_search(Scope, ServiceName, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
                                 {services_search, Scope, ServiceName,
                                  timeout_decr(Timeout)}, Timeout)).
 
+service_ids(Timeout) ->
+    ?CATCH_EXIT(gen_server:call(?MODULE, service_ids, Timeout)).
+
 services(Timeout) ->
-    ?CATCH_EXIT(gen_server:call(?MODULE,
-                                {services,
-                                 timeout_decr(Timeout)}, Timeout)).
+    ?CATCH_EXIT(gen_server:call(?MODULE, services, Timeout)).
 
 nodes_set(L, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
-                                {nodes_set, L, local,
-                                 timeout_decr(Timeout)}, Timeout)).
+                                {nodes_set, L, local}, Timeout)).
 
 nodes_get(Timeout) ->
-    ?CATCH_EXIT(gen_server:call(?MODULE,
-                                {nodes_get,
-                                 timeout_decr(Timeout)}, Timeout)).
+    ?CATCH_EXIT(gen_server:call(?MODULE, nodes_get, Timeout)).
 
 nodes_add(L, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
-                                {nodes_add, L, local,
-                                 timeout_decr(Timeout)}, Timeout)).
+                                {nodes_add, L, local}, Timeout)).
 nodes_remove(L, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
-                                {nodes_remove, L, local,
-                                 timeout_decr(Timeout)}, Timeout)).
+                                {nodes_remove, L, local}, Timeout)).
 
 logging_set(L, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
-                                {logging_set, L,
-                                 timeout_decr(Timeout)}, Timeout)).
+                                {logging_set, L}, Timeout)).
 
 logging_file_set(FilePath, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
-                                {logging_file_set, FilePath,
-                                 timeout_decr(Timeout)}, Timeout)).
+                                {logging_file_set, FilePath}, Timeout)).
 
 logging_stdout_set(Stdout, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
-                                {logging_stdout_set, Stdout,
-                                 timeout_decr(Timeout)}, Timeout)).
+                                {logging_stdout_set, Stdout}, Timeout)).
 
 logging_level_set(Level, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
-                                {logging_level_set, Level,
-                                 timeout_decr(Timeout)}, Timeout)).
+                                {logging_level_set, Level}, Timeout)).
 
 logging_syslog_set(L, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
-                                {logging_syslog_set, L,
-                                 timeout_decr(Timeout)}, Timeout)).
+                                {logging_syslog_set, L}, Timeout)).
 
 logging_formatters_set(L, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
-                                {logging_formatters_set, L,
-                                 timeout_decr(Timeout)}, Timeout)).
+                                {logging_formatters_set, L}, Timeout)).
 
 logging_redirect_set(L, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
-                                {logging_redirect_set, L,
-                                 timeout_decr(Timeout)}, Timeout)).
+                                {logging_redirect_set, L}, Timeout)).
 
 logging(Timeout) ->
+    ?CATCH_EXIT(gen_server:call(?MODULE, logging, Timeout)).
+
+code_status(TimeNative, TimeOffset, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
-                                {logging,
-                                 timeout_decr(Timeout)}, Timeout)).
+                                {code_status,
+                                 TimeNative, TimeOffset}, Timeout)).
 
 -spec service_start(#config_service_internal{} |
                     #config_service_external{},
@@ -339,21 +333,25 @@ service_update_external(Pids, Ports, Arguments,
     service_update_external(0, Pids, Ports, Arguments,
                             CountThread, CountProcess).
 
-service_initialized_process(Pid)
+service_process_init_end(Pid)
     when is_pid(Pid) ->
-    ?MODULE ! {service_initialized_process, Pid},
+    ?MODULE ! {cloudi_service_init_end, Pid},
     ok.
 
-service_dead(ID)
+-spec service_terminated(ID :: binary()) ->
+    ok.
+
+service_terminated(ID)
     when is_binary(ID) ->
-    gen_server:cast(?MODULE, {service_dead, ID}).
+    gen_server:cast(?MODULE, {service_terminated, ID}).
 
 %%%------------------------------------------------------------------------
 %%% Callback functions from gen_server
 %%%------------------------------------------------------------------------
 
 init([Config]) ->
-    {ok, #state{configuration = Config}}.
+    {ok, #state{time_start = cloudi_timestamp:native_monotonic(),
+                configuration = Config}}.
 
 handle_call(configure, _, State) ->
     % the application startup configuration must not block
@@ -361,7 +359,7 @@ handle_call(configure, _, State) ->
     self() ! configure,
     {reply, ok, State};
 
-handle_call({acl_add, L, _}, _,
+handle_call({acl_add, L}, _,
             #state{configuration = Config} = State) ->
     case cloudi_core_i_configuration:acl_add(L, Config) of
         {ok, NewConfig} ->
@@ -370,7 +368,7 @@ handle_call({acl_add, L, _}, _,
             {reply, Error, State}
     end;
 
-handle_call({acl_remove, L, _}, _,
+handle_call({acl_remove, L}, _,
             #state{configuration = Config} = State) ->
     case cloudi_core_i_configuration:acl_remove(L, Config) of
         {ok, NewConfig} ->
@@ -379,7 +377,7 @@ handle_call({acl_remove, L, _}, _,
             {reply, Error, State}
     end;
 
-handle_call({acl, _}, _,
+handle_call(acl, _,
             #state{configuration = Config} = State) ->
     {reply, {ok, cloudi_core_i_configuration:acl(Config)}, State};
 
@@ -393,36 +391,36 @@ handle_call({service_subscriptions, ServiceId, Timeout}, _, State) ->
             {reply, Error, State}
     end;
 
-handle_call({services_add, L, Timeout}, _,
+handle_call({services_add, L}, _,
             #state{configuration = Config} = State) ->
-    case cloudi_core_i_configuration:services_add(L, Config, Timeout) of
+    case cloudi_core_i_configuration:services_add(L, Config, infinity) of
         {ok, IDs, NewConfig} ->
             {reply, {ok, IDs}, State#state{configuration = NewConfig}};
         {error, _} = Error ->
             {reply, Error, State}
     end;
 
-handle_call({services_remove, L, Timeout}, _,
+handle_call({services_remove, L}, _,
             #state{configuration = Config} = State) ->
-    case cloudi_core_i_configuration:services_remove(L, Config, Timeout) of
+    case cloudi_core_i_configuration:services_remove(L, Config, infinity) of
         {ok, NewConfig} ->
             {reply, ok, State#state{configuration = NewConfig}};
         {error, _} = Error ->
             {reply, Error, State}
     end;
 
-handle_call({services_restart, L, Timeout}, _,
+handle_call({services_restart, L}, _,
             #state{configuration = Config} = State) ->
-    case cloudi_core_i_configuration:services_restart(L, Config, Timeout) of
+    case cloudi_core_i_configuration:services_restart(L, Config, infinity) of
         {ok, NewConfig} ->
             {reply, ok, State#state{configuration = NewConfig}};
         {error, _} = Error ->
             {reply, Error, State}
     end;
 
-handle_call({services_update, L, Timeout}, _,
+handle_call({services_update, L}, _,
             #state{configuration = Config} = State) ->
-    case cloudi_core_i_configuration:services_update(L, Config, Timeout) of
+    case cloudi_core_i_configuration:services_update(L, Config, infinity) of
         {ok, Result, NewConfig} ->
             {reply, Result, State#state{configuration = NewConfig}};
         {error, _} = Error ->
@@ -431,7 +429,7 @@ handle_call({services_update, L, Timeout}, _,
 
 handle_call({services_search, Scope, ServiceName, Timeout}, _,
             #state{configuration = Config} = State) ->
-    case cloudi_x_cpg:get_local_members(Scope, ServiceName, Timeout) of
+    try cloudi_x_cpg:get_local_members(Scope, ServiceName, Timeout) of
         {ok, _, PidList} ->
             case cloudi_core_i_services_monitor:search(PidList, Timeout) of
                 {ok, []} ->
@@ -446,26 +444,35 @@ handle_call({services_search, Scope, ServiceName, Timeout}, _,
             end;
         {error, _} ->
             {reply, {ok, []}, State}
+    catch
+        exit:{timeout, _} ->
+            {reply, {error, timeout}, State};
+        exit:{noproc, _} ->
+            {reply, {error, service_scope_invalid}, State}
     end;
 
-handle_call({services, _}, _,
+handle_call(service_ids, _,
+            #state{configuration = Config} = State) ->
+    {reply, {ok, cloudi_core_i_configuration:service_ids(Config)}, State};
+
+handle_call(services, _,
             #state{configuration = Config} = State) ->
     {reply, {ok, cloudi_core_i_configuration:services(Config)}, State};
 
-handle_call({nodes_set, _, _, _} = Request, _, State) ->
+handle_call({nodes_set, _, _} = Request, _, State) ->
     nodes_call(Request, State);
 
-handle_call({nodes_get, _}, _,
+handle_call(nodes_get, _,
             #state{configuration = Config} = State) ->
     {reply, {ok, cloudi_core_i_configuration:nodes_get(Config)}, State};
 
-handle_call({nodes_add, _, _, _} = Request, _, State) ->
+handle_call({nodes_add, _, _} = Request, _, State) ->
     nodes_call(Request, State);
 
-handle_call({nodes_remove, _, _, _} = Request, _, State) ->
+handle_call({nodes_remove, _, _} = Request, _, State) ->
     nodes_call(Request, State);
 
-handle_call({logging_set, L, _}, _,
+handle_call({logging_set, L}, _,
             #state{configuration = Config} = State) ->
     case cloudi_core_i_configuration:logging_set(L, Config) of
         {ok, #config{logging = LoggingConfig} = NewConfig} ->
@@ -479,7 +486,7 @@ handle_call({logging_set, L, _}, _,
             {reply, Error, State}
     end;
 
-handle_call({logging_file_set, FilePath, _}, _,
+handle_call({logging_file_set, FilePath}, _,
             #state{configuration = Config} = State) ->
     #config{logging = LoggingConfig} = Config,
     case cloudi_core_i_logger:file_set(FilePath) of
@@ -492,7 +499,7 @@ handle_call({logging_file_set, FilePath, _}, _,
             {reply, Error, State}
     end;
 
-handle_call({logging_stdout_set, Stdout, _}, _,
+handle_call({logging_stdout_set, Stdout}, _,
             #state{configuration = Config} = State) ->
     #config{logging = LoggingConfig} = Config,
     ok = cloudi_core_i_logger:stdout_set(Stdout),
@@ -501,7 +508,7 @@ handle_call({logging_stdout_set, Stdout, _}, _,
                         stdout = Stdout}},
     {reply, ok, State#state{configuration = NewConfig}};
 
-handle_call({logging_level_set, Level, _}, _,
+handle_call({logging_level_set, Level}, _,
             #state{configuration = Config} = State) ->
     #config{logging = LoggingConfig} = Config,
     ok = cloudi_core_i_logger:level_set(Level),
@@ -510,7 +517,7 @@ handle_call({logging_level_set, Level, _}, _,
                         level = Level}},
     {reply, ok, State#state{configuration = NewConfig}};
 
-handle_call({logging_syslog_set, L, _}, _,
+handle_call({logging_syslog_set, L}, _,
             #state{configuration = Config} = State) ->
     case cloudi_core_i_configuration:logging_syslog_set(L, Config) of
         {ok, #config{logging = #config_logging{
@@ -521,7 +528,7 @@ handle_call({logging_syslog_set, L, _}, _,
             {reply, Error, State}
     end;
 
-handle_call({logging_formatters_set, L, _}, _,
+handle_call({logging_formatters_set, L}, _,
             #state{configuration = Config} = State) ->
     case cloudi_core_i_configuration:logging_formatters_set(L, Config) of
         {ok, #config{logging = #config_logging{
@@ -532,7 +539,7 @@ handle_call({logging_formatters_set, L, _}, _,
             {reply, Error, State}
     end;
 
-handle_call({logging_redirect_set, Node, _}, _,
+handle_call({logging_redirect_set, Node}, _,
             #state{configuration = Config} = State) ->
     #config{logging = LoggingConfig} = Config,
     ok = cloudi_core_i_logger:redirect_set(Node),
@@ -541,31 +548,46 @@ handle_call({logging_redirect_set, Node, _}, _,
                         redirect = Node}},
     {reply, ok, State#state{configuration = NewConfig}};
 
-handle_call({logging, _}, _,
+handle_call(logging, _,
             #state{configuration = Config} = State) ->
     {reply, {ok, cloudi_core_i_configuration:logging(Config)}, State};
 
+handle_call({code_status, TimeNative, TimeOffset}, _,
+            #state{time_start = TimeStart,
+                   configuration = Config} = State) ->
+    #config{services = Services} = Config,
+    SecondsNow = cloudi_timestamp:convert(TimeNative + TimeOffset,
+                                          native, second),
+    SecondsStart = SecondsNow -
+                   cloudi_timestamp:convert(TimeNative - TimeStart,
+                                            native, second),
+    case code_status_files(Services, SecondsStart, SecondsNow) of
+        {ok, RuntimeChanges} ->
+            {reply, {ok, TimeStart, RuntimeChanges}, State};
+        {error, _} = Error ->
+            {reply, Error, State}
+    end;
+
 handle_call(Request, _, State) ->
-    ?LOG_WARN("Unknown call \"~p\"", [Request]),
-    {stop, cloudi_string:format("Unknown call \"~p\"", [Request]),
+    {stop, cloudi_string:format("Unknown call \"~w\"", [Request]),
      error, State}.
 
-handle_cast({service_dead, ID}, #state{configuration = Config} = State) ->
+handle_cast({service_terminated, ID},
+            #state{configuration = Config} = State) ->
     #config{services = Services} = Config,
     NewServices = lists:filter(fun(Service) ->
-        if
-            is_record(Service, config_service_internal) ->
-                Service#config_service_internal.uuid /= ID;
-            is_record(Service, config_service_external) ->
-                Service#config_service_external.uuid /= ID
+        case Service of
+            #config_service_internal{uuid = InternalID} ->
+                InternalID /= ID;
+            #config_service_external{uuid = ExternalID} ->
+                ExternalID /= ID
         end
     end, Services),
     NewConfig = Config#config{services = NewServices},
     {noreply, State#state{configuration = NewConfig}};
 
 handle_cast(Request, State) ->
-    ?LOG_WARN("Unknown cast \"~p\"", [Request]),
-    {noreply, State}.
+    {stop, cloudi_string:format("Unknown cast \"~w\"", [Request]), State}.
 
 handle_info(configure,
             #state{configuration = Config} = State) ->
@@ -578,8 +600,8 @@ handle_info(configure,
             {stop, Error, State}
     end;
 
-handle_info({service_initialized_process, _}, State) ->
-    % a service process has initialized after a restart occurred
+handle_info({cloudi_service_init_end, _}, State) ->
+    % a service process has initialized due to count_process_dynamic
     % (nothing to do, handled by cloudi_core_i_services_monitor)
     {noreply, State};
 
@@ -589,8 +611,7 @@ handle_info({ReplyRef, _}, State) when is_reference(ReplyRef) ->
     {noreply, State};
 
 handle_info(Request, State) ->
-    ?LOG_WARN("Unknown info \"~p\"", [Request]),
-    {noreply, State}.
+    {stop, cloudi_string:format("Unknown info \"~w\"", [Request]), State}.
 
 terminate(_, _) ->
     ok.
@@ -925,7 +946,7 @@ service_start_wait_pid([], Error) ->
     Error;
 service_start_wait_pid([{MonitorRef, Pid} | M], Error) ->
     receive
-        {service_initialized_process, Pid} ->
+        {cloudi_service_init_end, Pid} ->
             erlang:demonitor(MonitorRef, [flush]),
             % at this point:
             % - if it is an internal service process:
@@ -963,10 +984,9 @@ service_start_wait(Pids, Service) ->
     % and pass the error reason as a return value of the configure/2 function
     % or the cloudi_service_api:services_add/2 function when a service
     % starts for the first time
-    % (restarts require cloudi_core_i_services_monitor to send 'initialize')
-    lists:foreach(fun(P) ->
-        ok = cloudi_core_i_services_monitor:initialize(P)
-    end, Pids),
+    % (restarts require cloudi_core_i_services_monitor to
+    %  call cloudi_core_i_services_monitor:process_init_begin/1)
+    ok = cloudi_core_i_services_monitor:process_init_begin(Pids),
     service_start_wait_pids(MonitorPids, undefined, Service).
 
 service_start_internal(CountProcess, Pids, Service, _, CountProcess, _) ->
@@ -1167,9 +1187,9 @@ nodes_call_remote_result_replies([{_, ok} | Replies], Output) ->
 nodes_call_remote_result_replies([{_, _} = Error | Replies], Output) ->
     nodes_call_remote_result_replies(Replies, [Error | Output]).
 
-nodes_call_remote({_, _, remote, _}, _) ->
+nodes_call_remote({_, _, remote}, _) ->
     ok;
-nodes_call_remote({F, L, local, Timeout}, Connect) ->
+nodes_call_remote({F, L, local}, Connect) ->
     Nodes = if
         Connect =:= visible ->
             nodes();
@@ -1178,18 +1198,17 @@ nodes_call_remote({F, L, local, Timeout}, Connect) ->
     end,
     nodes_call_remote_result(global:trans({{?MODULE, L}, self()}, fun() ->
         gen_server:multi_call(Nodes, ?MODULE,
-                              {F, L, remote,
-                               timeout_decr(Timeout)}, Timeout)
+                              {F, L, remote}, infinity)
     end)).
 
-nodes_call({F, L, _, Timeout} = Request,
+nodes_call({F, L, _} = Request,
            #state{configuration = Config} = State) ->
     case cloudi_core_i_configuration:F(L, Config) of
         {ok, Config} ->
             {reply, ok, State};
         {ok, #config{nodes = #config_nodes{connect = Connect}} = NewConfig} ->
             Result = nodes_call_remote(Request, Connect),
-            case cloudi_core_i_nodes:reconfigure(NewConfig, Timeout) of
+            case cloudi_core_i_nodes:reconfigure(NewConfig, infinity) of
                 ok ->
                     {reply, Result, State#state{configuration = NewConfig}};
                 {error, _} = Error ->
@@ -1225,3 +1244,72 @@ service_format(Service) ->
 service_id(ID) ->
     cloudi_x_uuid:uuid_to_string(ID, list_nodash).
 
+code_status_files(Services, SecondsStart, SecondsNow) ->
+    code_status_files(Services, [], SecondsStart, SecondsNow).
+
+code_status_files([], ChangedFiles, _, _) ->
+    {ok,
+     [[{type, Type},
+       {file_age, cloudi_timestamp:seconds_to_string(SecondsElapsed)},
+       {file_path, FilePath},
+       {service_ids, IDs}]
+      || {SecondsElapsed, Type, FilePath, IDs} <- ChangedFiles]};
+code_status_files([Service | Services], ChangedFiles,
+                  SecondsStart, SecondsNow) ->
+    {Type, FilePath} = case Service of
+        #config_service_internal{file_path = FilePathValue}
+        when FilePathValue /= undefined ->
+            {internal, FilePathValue};
+        #config_service_internal{module = Module} ->
+            {file, FilePathValue} = code:is_loaded(Module),
+            {internal, FilePathValue};
+        #config_service_external{file_path = FilePathValue} ->
+            {external, FilePathValue}
+    end,
+    case read_file_info(FilePath) of
+        {ok, #file_info{mtime = MTime}} ->
+            {IDs, ServicesNew} = code_status_files_filter(Services, Service),
+            if
+                MTime > SecondsStart ->
+                    ChangedFile = {SecondsNow - MTime, Type, FilePath, IDs},
+                    code_status_files(ServicesNew,
+                                      lists:umerge(ChangedFiles,
+                                                   [ChangedFile]),
+                                      SecondsStart, SecondsNow);
+                true ->
+                    code_status_files(ServicesNew, ChangedFiles,
+                                      SecondsStart, SecondsNow)
+            end;
+        {error, Reason} ->
+            {error, {file, {FilePath, Reason}}}
+    end.
+
+code_status_files_filter(Services, S) ->
+    code_status_files_filter(Services, [], [], S).
+
+code_status_files_filter([], IDs, ServicesOut, S) ->
+    ID = case S of
+        #config_service_internal{uuid = IDValue} ->
+            IDValue;
+        #config_service_external{uuid = IDValue} ->
+            IDValue
+    end,
+    {[ID | lists:reverse(IDs)], lists:reverse(ServicesOut)};
+code_status_files_filter([#config_service_internal{module = Module,
+                                                   file_path = FilePath,
+                                                   uuid = ID} |
+                          ServicesIn], IDs, ServicesOut,
+                         #config_service_internal{module = Module,
+                                                  file_path = FilePath} = S) ->
+    code_status_files_filter(ServicesIn, [ID | IDs], ServicesOut, S);
+code_status_files_filter([#config_service_external{file_path = FilePath,
+                                                   uuid = ID} |
+                          ServicesIn], IDs, ServicesOut,
+                         #config_service_external{file_path = FilePath} = S) ->
+    code_status_files_filter(ServicesIn, [ID | IDs], ServicesOut, S);
+code_status_files_filter([Service | ServicesIn], IDs, ServicesOut, S) ->
+    code_status_files_filter(ServicesIn, IDs, [Service | ServicesOut], S).
+
+read_file_info(FilePath) ->
+    file:read_file_info(FilePath, [raw, {time, posix}]).
+
