@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2018 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2011-2018 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -28,20 +28,20 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
+%%% @author Michael Truog <mjtruog at protonmail dot com>
 %%% @copyright 2011-2018 Michael Truog
-%%% @version 1.7.3 {@date} {@time}
+%%% @version 1.7.4 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_spawn).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 %% external interface
 -export([environment_lookup/0,
          environment_transform/1,
          environment_transform/2,
-         start_internal/15,
-         start_external/18,
+         start_internal/18,
+         start_external/21,
          update_external/3,
          update_internal_f/9,
          update_external_f/12]).
@@ -50,8 +50,10 @@
 -include("cloudi_core_i_configuration.hrl").
 -include("cloudi_core_i_constants.hrl").
 
--define(CREATE_INTERNAL, cloudi_core_i_services_internal_sup:create_internal).
--define(CREATE_EXTERNAL, cloudi_core_i_services_external_sup:create_external).
+-define(PROCESS_START_INTERNAL,
+        cloudi_core_i_services_internal_sup:process_start).
+-define(PROCESS_START_EXTERNAL,
+        cloudi_core_i_services_external_sup:process_start).
 
 % environmental variables used by CloudI API initialization
 -define(ENVIRONMENT_THREAD_COUNT,  "CLOUDI_API_INIT_THREAD_COUNT").
@@ -84,11 +86,12 @@ environment_transform(String) ->
 environment_transform(String, EnvironmentLookup) ->
     environment_transform(String, [], undefined, EnvironmentLookup).
 
-start_internal(ProcessIndex, ProcessCount, GroupLeader,
-               Module, Args, Timeout, Prefix,
+start_internal(ProcessIndex, ProcessCount, TimeStart, TimeRestart, Restarts,
+               GroupLeader, Module, Args, Timeout, Prefix,
                TimeoutAsync, TimeoutSync, TimeoutTerm, DestRefresh,
                DestListDeny, DestListAllow, ConfigOptions, ID)
     when is_integer(ProcessIndex), is_integer(ProcessCount),
+         is_integer(TimeStart), is_integer(Restarts),
          is_atom(Module), is_list(Args), is_integer(Timeout), is_list(Prefix),
          is_integer(TimeoutAsync), is_integer(TimeoutSync),
          is_integer(TimeoutTerm),
@@ -125,18 +128,20 @@ start_internal(ProcessIndex, ProcessCount, GroupLeader,
         {ok, NewTimeout} ->
             % Erlang application startup is asynchronous, so wait for the
             % module to be loaded or timeout
-            ?CREATE_INTERNAL(ProcessIndex, ProcessCount, GroupLeader,
-                             Module, Args, NewTimeout, Prefix,
-                             TimeoutAsync, TimeoutSync, TimeoutTerm,
-                             DestRefresh, DestDeny, DestAllow,
-                             ConfigOptions, ID);
+            ?PROCESS_START_INTERNAL(ProcessIndex, ProcessCount,
+                                    TimeStart, TimeRestart, Restarts,
+                                    GroupLeader, Module, Args,
+                                    NewTimeout, Prefix,
+                                    TimeoutAsync, TimeoutSync, TimeoutTerm,
+                                    DestRefresh, DestDeny, DestAllow,
+                                    ConfigOptions, ID);
         {error, Reason} ->
             ?LOG_ERROR("loading ~p failed: ~p", [Module, Reason]),
             {error, {service_internal_module_not_loaded, Module}}
     end.
 
-start_external(ProcessIndex, ProcessCount, ThreadsPerProcess,
-               Filename, Arguments, Environment,
+start_external(ProcessIndex, ProcessCount, TimeStart, TimeRestart, Restarts,
+               ThreadsPerProcess, Filename, Arguments, Environment,
                Protocol, BufferSize, Timeout, Prefix,
                TimeoutAsync, TimeoutSync, TimeoutTerm, DestRefresh,
                DestListDeny, DestListAllow, ConfigOptions, ID) ->
@@ -157,6 +162,9 @@ start_external(ProcessIndex, ProcessCount, ThreadsPerProcess,
             case start_external_threads(ThreadsPerProcess,
                                         ProcessIndex,
                                         ProcessCount,
+                                        TimeStart,
+                                        TimeRestart,
+                                        Restarts,
                                         CommandLine,
                                         Protocol, SocketPath,
                                         BufferSize, Timeout,
@@ -495,27 +503,33 @@ start_external_spawn_params_parse(Filename, Arguments, ConfigOptions,
     end.
 
 start_external_thread(ThreadsPerProcess, Pids, Ports, ThreadsPerProcess,
-                      _, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _) ->
+                      _, _, _, _, _, _, _, _, _, _,
+                      _, _, _, _, _, _, _, _, _) ->
     {ok, lists:reverse(Pids), lists:reverse(Ports)};
 
 start_external_thread(I, Pids, Ports, ThreadsPerProcess,
-                      ProcessIndex, ProcessCount, CommandLine,
-                      Protocol, SocketPath, BufferSize, Timeout,
+                      ProcessIndex, ProcessCount,
+                      TimeStart, TimeRestart, Restarts,
+                      CommandLine, Protocol, SocketPath, BufferSize, Timeout,
                       Prefix, TimeoutAsync, TimeoutSync, TimeoutTerm,
                       DestRefresh, DestDeny, DestAllow,
                       ConfigOptions, ID) ->
-    case ?CREATE_EXTERNAL(Protocol, SocketPath,
-                          I + ThreadsPerProcess * ProcessIndex,
-                          ProcessIndex, ProcessCount, CommandLine,
-                          BufferSize, Timeout,
-                          Prefix, TimeoutAsync, TimeoutSync, TimeoutTerm,
-                          DestRefresh, DestDeny, DestAllow,
-                          ConfigOptions, ID) of
+    case ?PROCESS_START_EXTERNAL(Protocol, SocketPath,
+                                 I + ThreadsPerProcess * ProcessIndex,
+                                 ProcessIndex, ProcessCount,
+                                 TimeStart, TimeRestart, Restarts,
+                                 CommandLine, BufferSize,
+                                 Timeout, Prefix,
+                                 TimeoutAsync, TimeoutSync, TimeoutTerm,
+                                 DestRefresh, DestDeny, DestAllow,
+                                 ConfigOptions, ID) of
         {ok, Pid, Port} ->
             start_external_thread(I + 1, [Pid | Pids], [Port | Ports],
                                   ThreadsPerProcess,
-                                  ProcessIndex, ProcessCount, CommandLine,
-                                  Protocol, SocketPath, BufferSize, Timeout,
+                                  ProcessIndex, ProcessCount,
+                                  TimeStart, TimeRestart, Restarts,
+                                  CommandLine, Protocol, SocketPath,
+                                  BufferSize, Timeout,
                                   Prefix, TimeoutAsync, TimeoutSync,
                                   TimeoutTerm,
                                   DestRefresh, DestDeny, DestAllow,
@@ -526,14 +540,17 @@ start_external_thread(I, Pids, Ports, ThreadsPerProcess,
     end.
 
 start_external_threads(ThreadsPerProcess,
-                       ProcessIndex, ProcessCount, CommandLine,
-                       Protocol, SocketPath, BufferSize, Timeout,
+                       ProcessIndex, ProcessCount,
+                       TimeStart, TimeRestart, Restarts,
+                       CommandLine, Protocol, SocketPath, BufferSize, Timeout,
                        Prefix, TimeoutAsync, TimeoutSync, TimeoutTerm,
                        DestRefresh, DestDeny, DestAllow,
                        ConfigOptions, ID) ->
     start_external_thread(0, [], [], ThreadsPerProcess,
-                          ProcessIndex, ProcessCount, CommandLine,
-                          Protocol, SocketPath, BufferSize, Timeout,
+                          ProcessIndex, ProcessCount,
+                          TimeStart, TimeRestart, Restarts,
+                          CommandLine, Protocol, SocketPath,
+                          BufferSize, Timeout,
                           Prefix, TimeoutAsync, TimeoutSync, TimeoutTerm,
                           DestRefresh, DestDeny, DestAllow,
                           ConfigOptions, ID).
