@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2009-2018 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2009-2018 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -28,13 +28,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
+%%% @author Michael Truog <mjtruog at protonmail dot com>
 %%% @copyright 2009-2018 Michael Truog
-%%% @version 1.7.3 {@date} {@time}
+%%% @version 1.7.4 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_string).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 %% external interface
 -export([afterl/2,
@@ -63,7 +63,9 @@
          splitr/2,
          splitr/3,
          term_to_binary/1,
+         term_to_binary_compact/1,
          term_to_list/1,
+         term_to_list_compact/1,
          titlecase/1,
          trim/1,
          trim/2,
@@ -75,6 +77,12 @@
 
 % based on unicode_util:whitespace/0
 -define(WHITESPACE, [13,9,10,11,12,13,32,133,8206,8207,8232,8233]).
+% keep output in a single line when using ~p (printable) formatting
+-ifdef(ERLANG_OTP_VERSION_21_FEATURES).
+-define(COMPACT_LIMIT, "0").
+-else.
+-define(COMPACT_LIMIT, "1000000000000").
+-endif.
 
 -include("cloudi_core_i_constants.hrl").
 
@@ -418,6 +426,8 @@ findr(SearchPattern, String) ->
              A :: list()) ->
     string().
 
+-compile({inline, [{format, 2}]}).
+
 format(L, A) ->
     format_to_list(L, A).
 
@@ -663,6 +673,19 @@ term_to_binary(T) ->
     unicode:characters_to_binary(io_lib:format("~w", [T])).
 -endif.
 
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Convert an Erlang term to a compact binary string.===
+%% Output is a utf8 encoded binary.
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec term_to_binary_compact(T :: any()) ->
+    binary().
+
+term_to_binary_compact(T) ->
+    unicode:characters_to_binary(io_lib:format("~" ?COMPACT_LIMIT "p", [T])).
+
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Convert an Erlang term to a string.===
@@ -680,6 +703,18 @@ term_to_list(T) ->
     format("~w", [T]).
 -endif.
 
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Convert an Erlang term to a compact string.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec term_to_list_compact(T :: any()) ->
+    string().
+
+term_to_list_compact(T) ->
+    format("~" ?COMPACT_LIMIT "p", [T]).
+
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Return the string in titlecase.===
