@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2015-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2015-2018 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -28,13 +28,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2015-2017 Michael Truog
-%%% @version 1.7.3 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2015-2018 Michael Truog
+%%% @version 1.7.4 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_timestamp).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 %% external interface
 -export([convert/3,
@@ -43,17 +43,24 @@
          native_monotonic/0,
          native_os/0,
          seconds/0,
+         seconds_epoch_to_string/1,
          seconds_monotonic/0,
          seconds_os/0,
+         seconds_to_string/1,
+         seconds_to_string/2,
          milliseconds/0,
+         milliseconds_epoch_to_string/1,
          milliseconds_monotonic/0,
          milliseconds_os/0,
          microseconds/0,
+         microseconds_epoch_to_string/1,
          microseconds_monotonic/0,
          microseconds_os/0,
          nanoseconds/0,
          nanoseconds_monotonic/0,
          nanoseconds_os/0,
+         nanoseconds_to_string/1,
+         nanoseconds_to_string/2,
          seconds_filter/3,
          seconds_filter_monotonic/3,
          uptime/0,
@@ -64,7 +71,7 @@
 
 -type time_unit() :: second | millisecond | microsecond | nanosecond |
                      native | perf_counter | pos_integer().
-% UNIX epoch (1970-01-01T00:00:00) offsets (POSIX time)
+% UNIX epoch (1970-01-01T00:00:00Z) offsets (POSIX time)
 -type seconds_epoch() :: non_neg_integer().
 -type milliseconds_epoch() :: non_neg_integer().
 -type microseconds_epoch() :: non_neg_integer().
@@ -74,6 +81,7 @@
 -type milliseconds_monotonic() :: integer().
 -type microseconds_monotonic() :: integer().
 -type nanoseconds_monotonic() :: integer().
+-type native_monotonic() :: integer().
 -export_type([time_unit/0,
               seconds_epoch/0,
               milliseconds_epoch/0,
@@ -82,7 +90,8 @@
               seconds_monotonic/0,
               milliseconds_monotonic/0,
               microseconds_monotonic/0,
-              nanoseconds_monotonic/0]).
+              nanoseconds_monotonic/0,
+              native_monotonic/0]).
 
 -include("cloudi_core_i_constants.hrl").
 
@@ -149,7 +158,7 @@ timestamp() ->
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Native time units since the UNIX epoch.===
-%% (The UNIX epoch is 1970-01-01T00:00:00)
+%% (The UNIX epoch is 1970-01-01T00:00:00Z)
 %% @end
 %%-------------------------------------------------------------------------
 
@@ -164,7 +173,7 @@ native() ->
 %% @end
 %%-------------------------------------------------------------------------
 
--spec native_monotonic() -> integer().
+-spec native_monotonic() -> native_monotonic().
 
 native_monotonic() ->
     erlang:monotonic_time().
@@ -173,7 +182,7 @@ native_monotonic() ->
 %% @doc
 %% ===Native time units since the UNIX epoch, from the hardware.===
 %% Always prefer the mative function instead of this function.
-%% (The UNIX epoch is 1970-01-01T00:00:00)
+%% (The UNIX epoch is 1970-01-01T00:00:00Z)
 %% @end
 %%-------------------------------------------------------------------------
 
@@ -185,7 +194,7 @@ native_os() ->
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Seconds since the UNIX epoch.===
-%% (The UNIX epoch is 1970-01-01T00:00:00)
+%% (The UNIX epoch is 1970-01-01T00:00:00Z)
 %% @end
 %%-------------------------------------------------------------------------
 
@@ -199,6 +208,19 @@ seconds() ->
     erlang:system_time(seconds).
 -endif.
 
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Create an ISO8601 timestamp from seconds since the UNIX epoch.===
+%% (The UNIX epoch is 1970-01-01T00:00:00Z)
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec seconds_epoch_to_string(Seconds :: seconds_epoch()) ->
+    string().
+
+seconds_epoch_to_string(Seconds) ->
+    cloudi_core_i_logger:seconds_to_string(Seconds).
+
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Seconds since an undefined point in time, from the Erlang VM.===
@@ -219,7 +241,7 @@ seconds_monotonic() ->
 %% @doc
 %% ===Seconds since the UNIX epoch, from the hardware.===
 %% Always prefer the seconds function instead of this function.
-%% (The UNIX epoch is 1970-01-01T00:00:00)
+%% (The UNIX epoch is 1970-01-01T00:00:00Z)
 %% @end
 %%-------------------------------------------------------------------------
 
@@ -233,10 +255,74 @@ seconds_os() ->
     os:system_time(seconds).
 -endif.
 
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Format a seconds duration as a minimal string with lower-precision integers.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec seconds_to_string(TotalSeconds :: non_neg_integer()) ->
+    nonempty_string().
+
+seconds_to_string(TotalSeconds)
+    when is_integer(TotalSeconds), TotalSeconds >= 0 ->
+    Seconds = TotalSeconds rem ?SECONDS_IN_HOUR,
+    TotalHours = TotalSeconds div ?SECONDS_IN_HOUR,
+    Hours = TotalHours rem ?HOURS_IN_DAY,
+    TotalDays = TotalHours div ?HOURS_IN_DAY,
+    if
+        TotalDays > 0 ->
+            lists:flatten([time_value_to_list(TotalDays, "day"), $ ,
+                           time_value_to_list(Hours, "hour"), $ ,
+                           time_value_to_list(Seconds, "second")]);
+        Hours > 0 ->
+            lists:flatten([time_value_to_list(Hours, "hour"), $ ,
+                           time_value_to_list(Seconds, "second")]);
+        true ->
+            lists:flatten(time_value_to_list(Seconds, "second"))
+    end.
+
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Format a seconds duration with options as a minimal string with lower-precision integers.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec seconds_to_string(TotalSeconds :: integer(),
+                        Options :: signed) ->
+    nonempty_string().
+
+seconds_to_string(TotalSeconds, signed)
+    when is_integer(TotalSeconds) ->
+    {Sign, TotalSecondsPositive} = if
+        TotalSeconds >= 0 ->
+            {"+ ", TotalSeconds};
+        true ->
+            {"- ", -1 * TotalSeconds}
+    end,
+    Seconds = TotalSecondsPositive rem ?SECONDS_IN_HOUR,
+    TotalHours = TotalSecondsPositive div ?SECONDS_IN_HOUR,
+    Hours = TotalHours rem ?HOURS_IN_DAY,
+    TotalDays = TotalHours div ?HOURS_IN_DAY,
+    if
+        TotalDays > 0 ->
+            lists:flatten([Sign,
+                           time_value_to_list(TotalDays, "day"), $ ,
+                           time_value_to_list(Hours, "hour"), $ ,
+                           time_value_to_list(Seconds, "second")]);
+        Hours > 0 ->
+            lists:flatten([Sign,
+                           time_value_to_list(Hours, "hour"), $ ,
+                           time_value_to_list(Seconds, "second")]);
+        true ->
+            lists:flatten([Sign,
+                           time_value_to_list(Seconds, "second")])
+    end.
+
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Milliseconds since the UNIX epoch.===
-%% (The UNIX epoch is 1970-01-01T00:00:00)
+%% (The UNIX epoch is 1970-01-01T00:00:00Z)
 %% @end
 %%-------------------------------------------------------------------------
 
@@ -250,6 +336,19 @@ milliseconds() ->
     erlang:system_time(milli_seconds).
 -endif.
 
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Create an ISO8601 timestamp from milliseconds since the UNIX epoch.===
+%% (The UNIX epoch is 1970-01-01T00:00:00Z)
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec milliseconds_epoch_to_string(MilliSeconds :: milliseconds_epoch()) ->
+    string().
+
+milliseconds_epoch_to_string(MilliSeconds) ->
+    cloudi_core_i_logger:milliseconds_to_string(MilliSeconds).
+
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Milliseconds since an undefined point in time, from the Erlang VM.===
@@ -270,7 +369,7 @@ milliseconds_monotonic() ->
 %% @doc
 %% ===Milliseconds since the UNIX epoch, from the hardware.===
 %% Always prefer the milliseconds function instead of this function.
-%% (The UNIX epoch is 1970-01-01T00:00:00)
+%% (The UNIX epoch is 1970-01-01T00:00:00Z)
 %% @end
 %%-------------------------------------------------------------------------
 
@@ -287,7 +386,7 @@ milliseconds_os() ->
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Microseconds since the UNIX epoch.===
-%% (The UNIX epoch is 1970-01-01T00:00:00)
+%% (The UNIX epoch is 1970-01-01T00:00:00Z)
 %% @end
 %%-------------------------------------------------------------------------
 
@@ -301,6 +400,19 @@ microseconds() ->
     erlang:system_time(micro_seconds).
 -endif.
 
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Create an ISO8601 timestamp from microseconds since the UNIX epoch.===
+%% (The UNIX epoch is 1970-01-01T00:00:00Z)
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec microseconds_epoch_to_string(MicroSeconds :: microseconds_epoch()) ->
+    string().
+
+microseconds_epoch_to_string(MicroSeconds) ->
+    cloudi_core_i_logger:microseconds_to_string(MicroSeconds).
+
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Microseconds since an undefined point in time, from the Erlang VM.===
@@ -321,7 +433,7 @@ microseconds_monotonic() ->
 %% @doc
 %% ===Microseconds since the UNIX epoch, from the hardware.===
 %% Always prefer the microseconds function instead of this function.
-%% (The UNIX epoch is 1970-01-01T00:00:00)
+%% (The UNIX epoch is 1970-01-01T00:00:00Z)
 %% @end
 %%-------------------------------------------------------------------------
 
@@ -338,7 +450,7 @@ microseconds_os() ->
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Nanoseconds since the UNIX epoch.===
-%% (The UNIX epoch is 1970-01-01T00:00:00)
+%% (The UNIX epoch is 1970-01-01T00:00:00Z)
 %% @end
 %%-------------------------------------------------------------------------
 
@@ -372,7 +484,7 @@ nanoseconds_monotonic() ->
 %% @doc
 %% ===Nanoseconds since the UNIX epoch, from the hardware.===
 %% Always prefer the nanoseconds function instead of this function.
-%% (The UNIX epoch is 1970-01-01T00:00:00)
+%% (The UNIX epoch is 1970-01-01T00:00:00Z)
 %% @end
 %%-------------------------------------------------------------------------
 
@@ -386,6 +498,87 @@ nanoseconds_os() ->
     os:system_time(nano_seconds).
 -endif.
 
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Format a nanoseconds duration as a minimal string with lower-precision integers.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec nanoseconds_to_string(TotalNanoSeconds :: non_neg_integer()) ->
+    nonempty_string().
+
+nanoseconds_to_string(TotalNanoSeconds)
+    when is_integer(TotalNanoSeconds), TotalNanoSeconds >= 0 ->
+    NanoSeconds = TotalNanoSeconds rem ?NANOSECONDS_IN_SECOND,
+    TotalSeconds = TotalNanoSeconds div ?NANOSECONDS_IN_SECOND,
+    Seconds = TotalSeconds rem ?SECONDS_IN_HOUR,
+    TotalHours = TotalSeconds div ?SECONDS_IN_HOUR,
+    Hours = TotalHours rem ?HOURS_IN_DAY,
+    TotalDays = TotalHours div ?HOURS_IN_DAY,
+    if
+        TotalDays > 0 ->
+            lists:flatten([time_value_to_list(TotalDays, "day"), $ ,
+                           time_value_to_list(Hours, "hour"), $ ,
+                           time_value_to_list(Seconds, "second"), $ ,
+                           time_value_to_list(NanoSeconds, "nanosecond")]);
+        Hours > 0 ->
+            lists:flatten([time_value_to_list(Hours, "hour"), $ ,
+                           time_value_to_list(Seconds, "second"), $ ,
+                           time_value_to_list(NanoSeconds, "nanosecond")]);
+        Seconds > 0 ->
+            lists:flatten([time_value_to_list(Seconds, "second"), $ ,
+                           time_value_to_list(NanoSeconds, "nanosecond")]);
+        true ->
+            lists:flatten(time_value_to_list(NanoSeconds, "nanosecond"))
+    end.
+
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Format a nanoseconds duration with options as a minimal string with lower-precision integers.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec nanoseconds_to_string(TotalNanoSeconds :: integer(),
+                            Options :: signed) ->
+    nonempty_string().
+
+nanoseconds_to_string(TotalNanoSeconds, signed)
+    when is_integer(TotalNanoSeconds) ->
+    {Sign, TotalNanoSecondsPositive} = if
+        TotalNanoSeconds >= 0 ->
+            {"+ ", TotalNanoSeconds};
+        true ->
+            {"- ", -1 * TotalNanoSeconds}
+    end,
+    NanoSeconds = TotalNanoSecondsPositive rem ?NANOSECONDS_IN_SECOND,
+    TotalSeconds = TotalNanoSecondsPositive div ?NANOSECONDS_IN_SECOND,
+    Seconds = TotalSeconds rem ?SECONDS_IN_HOUR,
+    TotalHours = TotalSeconds div ?SECONDS_IN_HOUR,
+    Hours = TotalHours rem ?HOURS_IN_DAY,
+    TotalDays = TotalHours div ?HOURS_IN_DAY,
+    if
+        TotalDays > 0 ->
+            lists:flatten([Sign,
+                           time_value_to_list(TotalDays, "day"), $ ,
+                           time_value_to_list(Hours, "hour"), $ ,
+                           time_value_to_list(Seconds, "second"), $ ,
+                           time_value_to_list(NanoSeconds, "nanosecond")]);
+        Hours > 0 ->
+            lists:flatten([Sign,
+                           time_value_to_list(Hours, "hour"), $ ,
+                           time_value_to_list(Seconds, "second"), $ ,
+                           time_value_to_list(NanoSeconds, "nanosecond")]);
+        Seconds > 0 ->
+            lists:flatten([Sign,
+                           time_value_to_list(Seconds, "second"), $ ,
+                           time_value_to_list(NanoSeconds, "nanosecond")]);
+        true ->
+            lists:flatten([Sign,
+                           time_value_to_list(NanoSeconds, "nanosecond")])
+    end;
+nanoseconds_to_string(_, _) ->
+    erlang:exit(badarg).
+
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Filter a list of seconds since the UNIX epoch.===
@@ -467,7 +660,7 @@ uptime() ->
 uptime(TimeUnit)
     when TimeUnit =:= second; TimeUnit =:= millisecond;
          TimeUnit =:= microsecond; TimeUnit =:= nanosecond ->
-    Value = erlang:monotonic_time() - erlang:system_info(start_time),
+    Value = native_monotonic() - erlang:system_info(start_time),
     convert(Value, native, TimeUnit).
 
 %%-------------------------------------------------------------------------
@@ -480,7 +673,7 @@ uptime(TimeUnit)
     float().
 
 uptime_days() ->
-    uptime(second) / (60 * 60 * 24).
+    uptime(second) / (?SECONDS_IN_HOUR * ?HOURS_IN_DAY).
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -492,7 +685,7 @@ uptime_days() ->
     float().
 
 uptime_months() ->
-    uptime_days() / (365.25 / 12).
+    uptime_days() / ?DAYS_IN_MONTH.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -504,9 +697,14 @@ uptime_months() ->
     float().
 
 uptime_years() ->
-    uptime_days() / 365.25.
+    uptime_days() / ?DAYS_IN_YEAR.
 
 %%%------------------------------------------------------------------------
 %%% Private functions
 %%%------------------------------------------------------------------------
 
+time_value_to_list(1, Label) ->
+    ["1 ", Label];
+time_value_to_list(I, Label) ->
+    [erlang:integer_to_list(I), $ , Label, $s].
+
