@@ -7,7 +7,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2014-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2014-2018 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -27,9 +27,9 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2014-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2014-2018 Michael Truog
+%%% @version 1.7.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_SUITE).
@@ -54,7 +54,8 @@
          end_per_testcase/2]).
 
 %% test callbacks
--export([t_service_internal_sync_1/1,
+-export([t_cloudi_code_status_1/1,
+         t_service_internal_sync_1/1,
          t_service_internal_sync_2/1,
          t_service_internal_sync_3/1,
          t_service_internal_async_1/1,
@@ -219,10 +220,9 @@ cloudi_service_handle_info(increment,
                            #state{count = Count} = State, _Dispatcher) ->
     {noreply, State#state{count = Count + 2}};
 cloudi_service_handle_info(Request, State, _Dispatcher) ->
-    {stop, {unexpected_info, Request}, State}.
+    {stop, cloudi_string:format("Unknown info \"~w\"", [Request]), State}.
 
-cloudi_service_terminate(_Reason, _Timeout,
-                         undefined) ->
+cloudi_service_terminate(_Reason, _Timeout, undefined) ->
     % cloudi_service_init/3 caused an exception
     ok;
 cloudi_service_terminate(terminate_sleep_test_1, _Timeout,
@@ -247,6 +247,10 @@ cloudi_service_terminate(_Reason, _Timeout,
 cloudi_service_terminate(_Reason, _Timeout,
                          #state{count = 60}) ->
     % t_service_internal_aspects_1/1 result
+    ok;
+cloudi_service_terminate(_Reason, _Timeout,
+                         #state{count = 100}) ->
+    % t_service_internal_update_1/1 result
     ok.
 
 %%%------------------------------------------------------------------------
@@ -254,11 +258,14 @@ cloudi_service_terminate(_Reason, _Timeout,
 %%%------------------------------------------------------------------------
 
 all() ->
-    [{group, service_internal_1},
+    [{group, cloudi_runtime_1},
+     {group, service_internal_1},
      {group, cloudi_modules_1}].
 
 groups() ->
-    [{service_internal_1, [],
+    [{cloudi_runtime_1, [sequence],
+      [t_cloudi_code_status_1]},
+     {service_internal_1, [sequence],
       [t_service_internal_sync_1,
        t_service_internal_sync_2,
        t_service_internal_sync_3,
@@ -272,7 +279,7 @@ groups() ->
        t_service_internal_terminate_4,
        t_service_internal_update_1,
        t_service_internal_log_1]},
-     {cloudi_modules_1, [],
+     {cloudi_modules_1, [parallel],
       [t_cloudi_args_type_1,
        t_cloudi_service_name_1]}].
 
@@ -302,12 +309,22 @@ end_per_group(_GroupName, Config) ->
 
 init_per_testcase(TestCase) ->
     error_logger:info_msg("~p init~n", [TestCase]),
-    error_logger:tty(false),
+    if
+        TestCase =:= t_cloudi_code_status_1 ->
+            ok;
+        true ->
+            error_logger:tty(false)
+    end,
     ?LOG_INFO("~p init", [TestCase]),
     ok.
 
 end_per_testcase(TestCase) ->
-    error_logger:tty(true),
+    if
+        TestCase =:= t_cloudi_code_status_1 ->
+            ok;
+        true ->
+            error_logger:tty(true)
+    end,
     error_logger:info_msg("~p end~n", [TestCase]),
     ?LOG_INFO("~p end", [TestCase]),
     ok.
@@ -483,6 +500,12 @@ end_per_testcase(TestCase, Config) ->
 %%% test cases
 %%%------------------------------------------------------------------------
 
+t_cloudi_code_status_1(_Config) ->
+    {ok, CodeStatus} = cloudi_service_api:code_status(infinity),
+    error_logger:info_msg("~p~n", [CodeStatus]),
+    ?LOG_INFO("~p", [CodeStatus]),
+    ok.
+
 t_service_internal_sync_1(_Config) ->
     % make sure synchronous sends work normally,
     % fail within the cloudi_service_init/3 callback with invalid_state,
@@ -523,7 +546,8 @@ t_service_internal_sync_1(_Config) ->
                                    ServiceName,
                                    ?REQUEST2),
     true = cloudi_x_uuid:is_v1(TransId2),
-    true = (TransId2 > TransId1),
+    true = (cloudi_x_uuid:get_v1_time(TransId2) >
+            cloudi_x_uuid:get_v1_time(TransId1)),
     {error,
      {service_internal_start_failed,
       {error, invalid_state}}} = cloudi_service_api:services_add([
@@ -590,6 +614,7 @@ t_service_internal_async_1(_Config) ->
         [{prefix, ?SERVICE_PREFIX1},
          {module, ?MODULE},
          {args, [{mode, init_send_async_recv}]},
+         {max_r, 0},
          {options, [{automatic_loading, false}]}]
         ], infinity),
     {{ok, TransId2},
@@ -607,13 +632,15 @@ t_service_internal_async_1(_Config) ->
                                    ?REQUEST2),
     true = (Timeout2 > (TimeoutMax - 1000)) andalso (Timeout2 =< TimeoutMax),
     true = cloudi_x_uuid:is_v1(TransId2),
-    true = (TransId2 > TransId1),
+    true = (cloudi_x_uuid:get_v1_time(TransId2) >
+            cloudi_x_uuid:get_v1_time(TransId1)),
     {error,
      {service_internal_start_failed,
       {error, invalid_state}}} = cloudi_service_api:services_add([
         [{prefix, ?SERVICE_PREFIX1},
          {module, ?MODULE},
          {args, [{mode, init_send_async_recv}]},
+         {max_r, 0},
          {options, [{automatic_loading, false}]}]
         ], infinity),
     ok.
@@ -640,9 +667,12 @@ t_service_internal_async_2(_Config) ->
      Context4} = cloudi:recv_async(Context3, <<0:128>>),
     {{ok, ?RESPONSE_INFO1, ?RESPONSE1, TransId4},
      Context5} = cloudi:recv_async(Context4, <<0:128>>),
-    true = (TransId1 < TransId2) andalso
-           (TransId2 < TransId3) andalso
-           (TransId3 < TransId4),
+    true = (cloudi_x_uuid:get_v1_time(TransId1) <
+            cloudi_x_uuid:get_v1_time(TransId2)) andalso
+           (cloudi_x_uuid:get_v1_time(TransId2) <
+            cloudi_x_uuid:get_v1_time(TransId3)) andalso
+           (cloudi_x_uuid:get_v1_time(TransId3) <
+            cloudi_x_uuid:get_v1_time(TransId4)),
     {{ok,
       [{'send_async', ServiceName, ServiceName, ?REQUEST_INFO1, ?REQUEST1,
         Timeout1, 0, TransId1, Self},
@@ -675,9 +705,12 @@ t_service_internal_async_2(_Config) ->
        {?RESPONSE_INFO1, ?RESPONSE1, TransId7},
        {?RESPONSE_INFO1, ?RESPONSE1, TransId8}]},
      Context8} = cloudi:recv_asyncs(Context7, TransIds),
-    true = (TransId5 < TransId6) andalso
-           (TransId6 < TransId7) andalso
-           (TransId7 < TransId8),
+    true = (cloudi_x_uuid:get_v1_time(TransId5) <
+            cloudi_x_uuid:get_v1_time(TransId6)) andalso
+           (cloudi_x_uuid:get_v1_time(TransId6) <
+            cloudi_x_uuid:get_v1_time(TransId7)) andalso
+           (cloudi_x_uuid:get_v1_time(TransId7) <
+            cloudi_x_uuid:get_v1_time(TransId8)),
     {{ok,
       [{'send_async', ServiceName, ServiceName, ?REQUEST_INFO1, ?REQUEST1,
         Timeout5, 0, TransId5, Self},
@@ -715,9 +748,12 @@ t_service_internal_async_3(_Config) ->
        {?RESPONSE_INFO1, ?RESPONSE1, TransId3},
        {?RESPONSE_INFO1, ?RESPONSE1, TransId4}]},
      _Context1} = cloudi:send_sync(Context0, ServiceName, ?REQUEST3),
-    true = (TransId1 < TransId2) andalso
-           (TransId2 < TransId3) andalso
-           (TransId3 < TransId4),
+    true = (cloudi_x_uuid:get_v1_time(TransId1) <
+            cloudi_x_uuid:get_v1_time(TransId2)) andalso
+           (cloudi_x_uuid:get_v1_time(TransId2) <
+            cloudi_x_uuid:get_v1_time(TransId3)) andalso
+           (cloudi_x_uuid:get_v1_time(TransId3) <
+            cloudi_x_uuid:get_v1_time(TransId4)),
     true = cloudi_x_uuid:is_v1(TransId1),
     true = cloudi_x_uuid:is_v1(TransId2),
     true = cloudi_x_uuid:is_v1(TransId3),
