@@ -60,8 +60,7 @@ cloudi_service_handle_request(_Type, _Name, _Pattern, _RequestInfo, _Request,
     {reply, ResponseInfo, Response, State}.
 
 cloudi_service_handle_info(Request, State, _Dispatcher) ->
-    ?LOG_WARN("Unknown info \"~p\"", [Request]),
-    {noreply, State}.
+    {stop, cloudi_string:format("Unknown info \"~w\"", [Request]), State}.
 
 cloudi_service_terminate(_Reason, _Timeout, _State) ->
     ok.
@@ -214,7 +213,7 @@ t_example_without_db_1(_Config) ->
     %    oauth_signature="74KNZJeDHnMBp0EMJ9ZHt%2FXKycU%3D"
     Name1 = "/initiate/post",
     RequestInfo1 = [
-        {<<"url-path">>, <<"/initiate">>}, % <- from cloudi_service_http_cowboy
+        {<<"url-path">>, <<"/initiate">>}, % <- from cloudi_service_http_cowboy1
         {<<"host">>, <<"photos.example.net">>},
         {<<"authorization">>,
          <<"OAuth realm=\"Photos\","
@@ -245,7 +244,7 @@ t_example_without_db_1(_Config) ->
     % https://photos.example.net/authorize?oauth_token=hh5s93j4hdidpola
     Name2 = "/authorize/get",
     RequestInfo2 = [
-        {<<"url-path">>, <<"/authorize">>}, % <- from cloudi_service_http_cowboy
+        {<<"url-path">>, <<"/authorize">>}, % <- from cloudi_service_http_cowboy1
         {<<"host">>, <<"photos.example.net">>}],
     Request2 = [{<<"oauth_token">>, <<"hh5s93j4hdidpola">>}],
 
@@ -275,7 +274,7 @@ t_example_without_db_1(_Config) ->
     %    oauth_signature="gKgrFCywp7rO0OXSjdot%2FIHF7IU%3D"
     Name3 = "/token/post",
     RequestInfo3 = [
-        {<<"url-path">>, <<"/token">>}, % <- from cloudi_service_http_cowboy
+        {<<"url-path">>, <<"/token">>}, % <- from cloudi_service_http_cowboy1
         {<<"host">>, <<"photos.example.net">>},
         {<<"authorization">>,
          <<"OAuth realm=\"Photos\","
@@ -359,7 +358,7 @@ t_example_with_db_1(_Config) ->
     %    oauth_signature="74KNZJeDHnMBp0EMJ9ZHt%2FXKycU%3D"
     Name1 = "/initiate/post",
     RequestInfo1 = [
-        {<<"url-path">>, <<"/initiate">>}, % <- from cloudi_service_http_cowboy
+        {<<"url-path">>, <<"/initiate">>}, % <- from cloudi_service_http_cowboy1
         {<<"host">>, <<"photos.example.net">>},
         {<<"authorization">>,
          <<"OAuth realm=\"Photos\","
@@ -388,7 +387,7 @@ t_example_with_db_1(_Config) ->
     % https://photos.example.net/authorize?oauth_token=hh5s93j4hdidpola
     Name2 = "/authorize/get",
     RequestInfo2 = [
-        {<<"url-path">>, <<"/authorize">>}, % <- from cloudi_service_http_cowboy
+        {<<"url-path">>, <<"/authorize">>}, % <- from cloudi_service_http_cowboy1
         {<<"host">>, <<"photos.example.net">>}],
     Request2 = [{<<"oauth_token">>, TokenRequest}],
 
@@ -431,7 +430,7 @@ t_example_with_db_1(_Config) ->
     %    oauth_signature="gKgrFCywp7rO0OXSjdot%2FIHF7IU%3D"
     Name3 = "/token/post",
     RequestInfo3 = [
-        {<<"url-path">>, <<"/token">>}, % <- from cloudi_service_http_cowboy
+        {<<"url-path">>, <<"/token">>}, % <- from cloudi_service_http_cowboy1
         {<<"host">>, <<"photos.example.net">>},
         {<<"authorization">>,
          <<(<<"OAuth realm=\"Photos\","
@@ -486,7 +485,7 @@ t_example_with_db_1(_Config) ->
     Name4 = "/verify/photos/get",
     RequestInfo4 = [
         {<<"url-path">>,
-         <<"/verify/photos">>}, % <- from cloudi_service_http_cowboy
+         <<"/verify/photos">>}, % <- from cloudi_service_http_cowboy1
         {<<"host">>, <<"photos.example.net">>},
         {<<"authorization">>,
          <<(<<"OAuth realm=\"Photos\","
