@@ -16,7 +16,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2013-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2013-2018 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -36,13 +36,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2013-2017 Michael Truog
-%%% @version 1.7.3 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2013-2018 Michael Truog
+%%% @version 1.7.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_quorum).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -behaviour(cloudi_service).
 
@@ -194,7 +194,7 @@ cloudi_service_init(Args, Prefix, _Timeout, Dispatcher) ->
     true = (FailuresDstMaxPeriod =:= infinity) orelse
            (is_integer(FailuresDstMaxPeriod) andalso
             (FailuresDstMaxPeriod > 0)),
-    false = cloudi_x_trie:is_pattern(Prefix),
+    false = cloudi_service_name:pattern(Prefix),
     cloudi_service:subscribe(Dispatcher, "*"),
     {ok, #state{quorum = Quorum,
                 use_response_info = UseResponseInfo,
@@ -493,10 +493,9 @@ cloudi_service_handle_info({'DOWN', _MonitorRef, process, Pid, _Info},
                           failures_dest = NewFailuresDst}};
 
 cloudi_service_handle_info(Request, State, _Dispatcher) ->
-    ?LOG_WARN("Unknown info \"~p\"", [Request]),
-    {noreply, State}.
+    {stop, cloudi_string:format("Unknown info \"~w\"", [Request]), State}.
 
-cloudi_service_terminate(_Reason, _Timeout, #state{}) ->
+cloudi_service_terminate(_Reason, _Timeout, _State) ->
     ok.
 
 %%%------------------------------------------------------------------------
