@@ -12,7 +12,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2018 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2011-2018 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -32,13 +32,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
+%%% @author Michael Truog <mjtruog at protonmail dot com>
 %%% @copyright 2011-2018 Michael Truog
-%%% @version 1.7.3 {@date} {@time}
+%%% @version 1.7.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cpg_data).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -export([get_groups/0,
          get_groups/1,
@@ -895,12 +895,12 @@ get_remote_newest_pid(GroupName, Exclude, Groups)
 %%% Private functions
 %%%------------------------------------------------------------------------
 
-% should names be matched with "*" interpreted as a wildcard within the
-% trie holding the groups of processes
+% should names be matched with "*" and "?" interpreted as wildcard characters
+% within the trie holding the groups of processes
 
 % matching with patterns
 group_find(GroupName, {trie, GroupsData}) ->
-    try trie:find_match(GroupName, GroupsData) catch
+    try trie:find_match2(GroupName, GroupsData) catch
         exit:badarg ->
             error
     end;
