@@ -2,45 +2,28 @@
 #-*-Mode:ruby;coding:utf-8;tab-width:4;c-basic-offset:4;indent-tabs-mode:()-*-
 # ex: set ft=ruby fenc=utf-8 sts=4 ts=4 sw=4 et nomod:
 #
-# BSD LICENSE
-# 
-# Copyright (c) 2012-2014, Michael Truog <mjtruog at gmail dot com>
-# All rights reserved.
-# 
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions are met:
-# 
-#     * Redistributions of source code must retain the above copyright
-#       notice, this list of conditions and the following disclaimer.
-#     * Redistributions in binary form must reproduce the above copyright
-#       notice, this list of conditions and the following disclaimer in
-#       the documentation and/or other materials provided with the
-#       distribution.
-#     * All advertising materials mentioning features or use of this
-#       software must display the following acknowledgment:
-#         This product includes software developed by Michael Truog
-#     * The name of the author may not be used to endorse or promote
-#       products derived from this software without specific prior
-#       written permission
-# 
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-# DAMAGE.
+# MIT License
+#
+# Copyright (c) 2012-2017 Michael Truog <mjtruog at protonmail dot com>
+#
+# Permission is hereby granted, free of charge, to any person obtaining a
+# copy of this software and associated documentation files (the "Software"),
+# to deal in the Software without restriction, including without limitation
+# the rights to use, copy, modify, merge, publish, distribute, sublicense,
+# and/or sell copies of the Software, and to permit persons to whom the
+# Software is furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+# DEALINGS IN THE SOFTWARE.
 #
-
-path = File.split(File.dirname(__FILE__)); path.pop(2)
-$:.unshift File.join(*path, *%w[api ruby])
 
 $DEBUG = false
 
@@ -105,103 +88,113 @@ if __FILE__ == $PROGRAM_NAME
                 CloudI::API.assert(&test)
             end
 
-            def sequence1_abcd(command, name, pattern, request_info, request,
+            def sequence1_abcd(request_type, name, pattern,
+                               request_info, request,
                                timeout, priority, trans_id, pid)
                 assert{pattern == "#{@api.prefix}a/b/c/d"}
                 assert{request == 'test1'}
-                @api.return_(command, name, pattern,
+                @api.return_(request_type, name, pattern,
                              '', request, timeout, trans_id, pid)
             end
 
-            def sequence1_abc_(command, name, pattern, request_info, request,
+            def sequence1_abc_(request_type, name, pattern,
+                               request_info, request,
                                timeout, priority, trans_id, pid)
                 assert{pattern == "#{@api.prefix}a/b/c/*"}
                 assert{request == 'test2' or request == 'test3'}
-                @api.return_(command, name, pattern,
+                @api.return_(request_type, name, pattern,
                              '', request, timeout, trans_id, pid)
             end
 
-            def sequence1_ab_d(command, name, pattern, request_info, request,
+            def sequence1_ab_d(request_type, name, pattern,
+                               request_info, request,
                                timeout, priority, trans_id, pid)
                 assert{pattern == "#{@api.prefix}a/b/*/d"}
                 assert{request == 'test4' or request == 'test5'}
-                @api.return_(command, name, pattern,
+                @api.return_(request_type, name, pattern,
                              '', request, timeout, trans_id, pid)
             end
 
-            def sequence1_a_cd(command, name, pattern, request_info, request,
+            def sequence1_a_cd(request_type, name, pattern,
+                               request_info, request,
                                timeout, priority, trans_id, pid)
                 assert{pattern == "#{@api.prefix}a/*/c/d"}
                 assert{request == 'test6' or request == 'test7'}
-                @api.return_(command, name, pattern,
+                @api.return_(request_type, name, pattern,
                              '', request, timeout, trans_id, pid)
             end
 
-            def sequence1__bcd(command, name, pattern, request_info, request,
+            def sequence1__bcd(request_type, name, pattern,
+                               request_info, request,
                                timeout, priority, trans_id, pid)
                 assert{pattern == "#{@api.prefix}*/b/c/d"}
                 assert{request == 'test8' or request == 'test9'}
-                @api.return_(command, name, pattern,
+                @api.return_(request_type, name, pattern,
                              '', request, timeout, trans_id, pid)
             end
 
-            def sequence1_ab__(command, name, pattern, request_info, request,
+            def sequence1_ab__(request_type, name, pattern,
+                               request_info, request,
                                timeout, priority, trans_id, pid)
                 assert{pattern == "#{@api.prefix}a/b/*"}
                 assert{request == 'test10'}
-                @api.return_(command, name, pattern,
+                @api.return_(request_type, name, pattern,
                              '', request, timeout, trans_id, pid)
             end
 
-            def sequence1_a__d(command, name, pattern, request_info, request,
+            def sequence1_a__d(request_type, name, pattern,
+                               request_info, request,
                                timeout, priority, trans_id, pid)
                 assert{pattern == "#{@api.prefix}a/*/d"}
                 assert{request == 'test11'}
-                @api.return_(command, name, pattern,
+                @api.return_(request_type, name, pattern,
                              '', request, timeout, trans_id, pid)
             end
 
-            def sequence1___cd(command, name, pattern, request_info, request,
+            def sequence1___cd(request_type, name, pattern,
+                               request_info, request,
                                timeout, priority, trans_id, pid)
                 assert{pattern == "#{@api.prefix}*/c/d"}
                 assert{request == 'test12'}
-                @api.return_(command, name, pattern,
+                @api.return_(request_type, name, pattern,
                              '', request, timeout, trans_id, pid)
             end
 
-            def sequence1_a___(command, name, pattern, request_info, request,
+            def sequence1_a___(request_type, name, pattern,
+                               request_info, request,
                                timeout, priority, trans_id, pid)
                 assert{pattern == "#{@api.prefix}a/*"}
                 assert{request == 'test13'}
-                @api.return_(command, name, pattern,
+                @api.return_(request_type, name, pattern,
                              '', request, timeout, trans_id, pid)
             end
 
-            def sequence1____d(command, name, pattern, request_info, request,
+            def sequence1____d(request_type, name, pattern,
+                               request_info, request,
                                timeout, priority, trans_id, pid)
                 assert{pattern == "#{@api.prefix}*/d"}
                 assert{request == 'test14'}
-                @api.return_(command, name, pattern,
+                @api.return_(request_type, name, pattern,
                              '', request, timeout, trans_id, pid)
             end
 
-            def sequence1_____(command, name, pattern, request_info, request,
+            def sequence1_____(request_type, name, pattern,
+                               request_info, request,
                                timeout, priority, trans_id, pid)
                 assert{pattern == "#{@api.prefix}*"}
                 assert{request == 'test15'}
-                @api.return_(command, name, pattern,
+                @api.return_(request_type, name, pattern,
                              '', request, timeout, trans_id, pid)
             end
 
-            def sequence1(command, name, pattern, request_info, request,
+            def sequence1(request_type, name, pattern,
+                          request_info, request,
                           timeout, priority, trans_id, pid)
                 while @api.recv_async(1000)[1] == 'end'
                     # consume "end" and sleep
                 end
                 $stdout.puts 'messaging sequence1 start ruby'
                 assert{request == 'start'}
-                # n.b., depends on cloudi_constants.hrl having
-                # SERVICE_NAME_PATTERN_MATCHING defined
                 test1_id = @api.send_async("#{@api.prefix}a/b/c/d", 'test1')
                 test2_id = @api.send_async("#{@api.prefix}a/b/c/z", 'test2')
                 test3_id = @api.send_async("#{@api.prefix}a/b/c/dd", 'test3')
@@ -217,7 +210,7 @@ if __FILE__ == $PROGRAM_NAME
                 test13_id = @api.send_async("#{@api.prefix}a/bzczd", 'test13')
                 test14_id = @api.send_async("#{@api.prefix}azbzc/d", 'test14')
                 test15_id = @api.send_async("#{@api.prefix}azbzczd", 'test15')
-                # n.b., depends on cloudi_constants.hrl having
+                # n.b., depends on cloudi_core_i_constants.hrl having
                 # RECV_ASYNC_STRATEGY == recv_async_select_oldest
                 @api.recv_async(nil, test1_id, false)
                 test1 = @api.recv_async()
@@ -312,59 +305,68 @@ if __FILE__ == $PROGRAM_NAME
                 $stdout.puts 'messaging sequence1 end ruby'
                 # start sequence2
                 @api.send_async("#{@api.prefix}sequence2", 'start')
-                @api.return_(command, name, pattern,
+                @api.return_(request_type, name, pattern,
                              '', 'end', timeout, trans_id, pid)
             end
 
-            def sequence2_e1(command, name, pattern, request_info, request,
+            def sequence2_e1(request_type, name, pattern,
+                             request_info, request,
                              timeout, priority, trans_id, pid)
-                @api.return_(command, name, pattern,
+                @api.return_(request_type, name, pattern,
                              '', '1', timeout, trans_id, pid)
             end
 
-            def sequence2_e2(command, name, pattern, request_info, request,
+            def sequence2_e2(request_type, name, pattern,
+                             request_info, request,
                              timeout, priority, trans_id, pid)
-                @api.return_(command, name, pattern,
+                @api.return_(request_type, name, pattern,
                              '', '2', timeout, trans_id, pid)
             end
 
-            def sequence2_e3(command, name, pattern, request_info, request,
+            def sequence2_e3(request_type, name, pattern,
+                             request_info, request,
                              timeout, priority, trans_id, pid)
-                @api.return_(command, name, pattern,
+                @api.return_(request_type, name, pattern,
                              '', '3', timeout, trans_id, pid)
             end
 
-            def sequence2_e4(command, name, pattern, request_info, request,
+            def sequence2_e4(request_type, name, pattern,
+                             request_info, request,
                              timeout, priority, trans_id, pid)
-                @api.return_(command, name, pattern,
+                @api.return_(request_type, name, pattern,
                              '', '4', timeout, trans_id, pid)
             end
 
-            def sequence2_e5(command, name, pattern, request_info, request,
+            def sequence2_e5(request_type, name, pattern,
+                             request_info, request,
                              timeout, priority, trans_id, pid)
-                @api.return_(command, name, pattern,
+                @api.return_(request_type, name, pattern,
                              '', '5', timeout, trans_id, pid)
             end
 
-            def sequence2_e6(command, name, pattern, request_info, request,
+            def sequence2_e6(request_type, name, pattern,
+                             request_info, request,
                              timeout, priority, trans_id, pid)
-                @api.return_(command, name, pattern,
+                @api.return_(request_type, name, pattern,
                              '', '6', timeout, trans_id, pid)
             end
 
-            def sequence2_e7(command, name, pattern, request_info, request,
+            def sequence2_e7(request_type, name, pattern,
+                             request_info, request,
                              timeout, priority, trans_id, pid)
-                @api.return_(command, name, pattern,
+                @api.return_(request_type, name, pattern,
                              '', '7', timeout, trans_id, pid)
             end
 
-            def sequence2_e8(command, name, pattern, request_info, request,
+            def sequence2_e8(request_type, name, pattern,
+                             request_info, request,
                              timeout, priority, trans_id, pid)
-                @api.return_(command, name, pattern,
+                @api.return_(request_type, name, pattern,
                              '', '8', timeout, trans_id, pid)
             end
 
-            def sequence2(command, name, pattern, request_info, request,
+            def sequence2(request_type, name, pattern,
+                          request_info, request,
                           timeout, priority, trans_id, pid)
                 $stdout.puts 'messaging sequence2 start ruby'
                 assert{request == 'start'}
@@ -390,38 +392,42 @@ if __FILE__ == $PROGRAM_NAME
                 $stdout.puts 'messaging sequence2 end ruby'
                 # start sequence3
                 @api.send_async("#{@api.prefix}sequence3", 'start')
-                @api.return_(command, name, pattern,
+                @api.return_(request_type, name, pattern,
                              '', 'end', timeout, trans_id, pid)
             end
 
-            def sequence3_f1(command, name, pattern, request_info, request,
+            def sequence3_f1(request_type, name, pattern,
+                             request_info, request,
                              timeout, priority, trans_id, pid)
                 request_i = request.to_i
                 if request_i == 4
                     return 'done'
                 end
                 request_new = request_i + 2 # two steps forward
-                @api.forward_(command, "#{@api.prefix}f2",
+                @api.forward_(request_type, "#{@api.prefix}f2",
                               '', request_new.to_s,
                               timeout, priority, trans_id, pid)
             end
 
-            def sequence3_f2(command, name, pattern, request_info, request,
+            def sequence3_f2(request_type, name, pattern,
+                             request_info, request,
                              timeout, priority, trans_id, pid)
                 request_i = request.to_i
                 request_new = request_i - 1 # one step back
-                @api.forward_(command, "#{@api.prefix}f1",
+                @api.forward_(request_type, "#{@api.prefix}f1",
                               '', request_new.to_s,
                               timeout, priority, trans_id, pid)
             end
 
-            def sequence3_g1(command, name, pattern, request_info, request,
+            def sequence3_g1(request_type, name, pattern,
+                             request_info, request,
                              timeout, priority, trans_id, pid)
-                @api.return_(command, name, pattern,
+                @api.return_(request_type, name, pattern,
                              '', request + 'suffix', timeout, trans_id, pid)
             end
 
-            def sequence3(command, name, pattern, request_info, request,
+            def sequence3(request_type, name, pattern,
+                          request_info, request,
                           timeout, priority, trans_id, pid)
                 $stdout.puts 'messaging sequence3 start ruby'
                 assert{request == 'start'}
@@ -437,7 +443,7 @@ if __FILE__ == $PROGRAM_NAME
                 $stdout.puts 'messaging sequence3 end ruby'
                 # loop to find any infrequent problems, restart sequence1
                 @api.send_async(@api.prefix + 'sequence1', 'start')
-                @api.return_(command, name, pattern,
+                @api.return_(request_type, name, pattern,
                              '', 'end', timeout, trans_id, pid)
             end
         end
